@@ -164,9 +164,16 @@ class LinuxDevice(object):
         #for k,v in default.iteritems():
         for k,v in default.items():
             self[k] = v
+            if kwargs.get("log_level", 0) >= 8:
+                print("default %s: %s" %(k, v))
         #for k,v in kwargs.iteritems():
         for k,v in kwargs.items():
             self[k] = v
+            if kwargs.get("log_level", 0) >= 8:
+                print("kwargs %s: %s" %(k, v))
+        # back compatiable with "prompt = xxx" in old release.
+        self["expect"] = self["expect"] and self["expect"] or self["prompt"]
+
         if server or self["url"]:
             # Python 2.
             u = urlparse.urlparse(server and server or self["url"])
@@ -196,7 +203,7 @@ class LinuxDevice(object):
         else:
             self.name = "Error"
 
-        if self.attributes.get("autologin", True):
+        if self["autologin"]:
             self.psession = self.login()
             self.connected = True
 
@@ -249,7 +256,7 @@ class LinuxDevice(object):
             self.log("%s\n" %(cmd))
         else:
             cmd = 'sh'
-        psession = pexpect.spawn("%s" %(cmd))
+        psession = pexpect.spawn("%s" %(cmd), encoding='utf-8')
         self.psession = psession
         if self["log_level"] >= LOG_VERBOSE:
             psession.logfile = sys.stdout
@@ -261,14 +268,16 @@ class LinuxDevice(object):
         prompt = ["(?i)Are you sure you want to continue connecting (yes/no/[fingerprint])?",
                 "(?i)(?:password)|(?:passphrase for key)",
                 "(?i)terminal type",
+                "Escape character is",
                 #"(?i)permission denied",
                 "(?i)connection closed by remote host",
                 pexpect.EOF,
                 pexpect.TIMEOUT,
-                "%s" %(self.attributes.get("prompt", False) and self["prompt"] or self.original_prompt)]
+                "%s" %(self["expect"] and self["expect"] or self.original_prompt)]
 
+        self.log("prompt: %s" %(prompt), level = LOG_DEBUG)
         i = psession.expect(prompt, timeout = login_timeout)
-        self.log("login phase 1 match: %s\n" %(prompt[i]), level = LOG_DEBUG)
+        self.log("login phase 1 match: %d/%s\n" %(i, prompt[i]), level = LOG_DEBUG)
         # First phase
         if i==0: 
             # New certificate -- always accept it.
@@ -280,8 +289,12 @@ class LinuxDevice(object):
         if i==2:
             psession.sendline(terminal_type)
             i = psession.expect(prompt)
-        self.log("login phase 2 match: %s\n" %(prompt[i]), level = LOG_DEBUG)
-        if i != 6:
+        if i==3:
+            psession.sendline("")
+            i = psession.expect(prompt)
+
+        self.log("login phase 2 match: %d/%s\n" %(i, prompt[i]), level = LOG_DEBUG)
+        if i != 7:
             psession.close()
             error = (i < len(prompt)) and prompt[i] or "None"
             self.log('Login failed: %s' %(error), color = "red")
@@ -301,23 +314,27 @@ class LinuxDevice(object):
                 LinuxDevice.cmd(self, self["root_password"],
                         expect = self.original_prompt, log_level = LOG_DEBUG)
 
-        # SUBTLE HACK ALERT! Note that the command to set the prompt uses a
-        # slightly different string than the regular expression to match it. This
-        # is because when you set the prompt the command will echo back, but we
-        # don't want to match the echoed command. So if we make the set command
-        # slightly different than the regex we eliminate the problem. To make the
-        # set command different we add a backslash in front of $. The $ doesn't
-        # need to be escaped, but it doesn't hurt and serves to make the set
-        # prompt command different than the regex.
-        #
-        # if you 'su' to a different user then you will need to manually reset
-        # the prompt. This sends shell commands to the remote host to set the
-        # prompt, so this assumes the remote host is ready to receive commands. 
-        if not LinuxDevice.cmd(self, self.PROMPT_SET_SH, format = "status",
-                timeout=1, with_log = False, log_level = self.LOG_DEBUG):
-            LinuxDevice.cmd(self, self.PROMPT_SET_CSH, format = "status",
-                                timeout=1, with_log = False,
-                                log_level = self.LOG_DEBUG)
+        if self["expect"]:
+            # Use user provided prompt at high priority
+            self.prompt = [self["expect"]]
+        else:
+            # SUBTLE HACK ALERT! Note that the command to set the prompt uses a
+            # slightly different string than the regular expression to match it. This
+            # is because when you set the prompt the command will echo back, but we
+            # don't want to match the echoed command. So if we make the set command
+            # slightly different than the regex we eliminate the problem. To make the
+            # set command different we add a backslash in front of $. The $ doesn't
+            # need to be escaped, but it doesn't hurt and serves to make the set
+            # prompt command different than the regex.
+            #
+            # if you 'su' to a different user then you will need to manually reset
+            # the prompt. This sends shell commands to the remote host to set the
+            # prompt, so this assumes the remote host is ready to receive commands. 
+            if not LinuxDevice.cmd(self, self.PROMPT_SET_SH, format = "status",
+                    timeout=1, with_log = False, log_level = self.LOG_DEBUG):
+                LinuxDevice.cmd(self, self.PROMPT_SET_CSH, format = "status",
+                                    timeout=1, with_log = False,
+                                    log_level = self.LOG_DEBUG)
         self.connected = True
         return psession
 
@@ -461,9 +478,9 @@ class LinuxDevice(object):
         # Prepare the output. Strip the line beofre the first '\r\n" since it
         # must be a command itself. Will correct it if encounter issue.
         #len1 = self.psession.before.find('%s' %(cmd))
-        len1 = self.psession.before.decode("utf-8").find("\r\n")
-        len2 = self.psession.before.decode("utf-8").rfind("\r\n")
-        output = self.psession.before[len1:len2].decode("utf-8").strip("\r\n")
+        len1 = self.psession.before.find("\r\n")
+        len2 = self.psession.before.rfind("\r\n")
+        output = self.psession.before[len1:len2].strip("\r\n")
         # It's a bug workaround for the issues/669:
         output = re.sub(r"\x1b\[\?2004l\r", "", output)
         #print("cmd: %s, output: %s" %(cmd, output))
